
/* Sx cpu micro architecture simulator

   5 December 2001
   modify from s2m sim
   Computer Architecture 1997, 1998
   microprogram version 		13 Feb 2005

   sx micro						10 July 2006

   P. Chongstitvatana
   Department of Computer Engineering
   Chulalongkorn University
*/

#include "sx.h"

int TS, FP, SP, NX, FF, IR, PC, Z, S, M[MAXMEM];
int runflag;

int icnt[100], lcnt[10000];
extern int CP, DP, tok;
extern int intflag, dt;

char opstr[][8] = {
  "Nop","Add","Sub","Mul","Div","Band","Bor","Bxor","Not","Eq",
  "Ne","Lt","Le","Ge","Gt","Shl","Shr","Mod","Ldx","Stx",
  "Ret","Retv","Array","End","Get","Put","Ld","St","Jmp","Jt",
  "Jf","Lit","Call","?","Inc","Dec","Sys","Case","Fun"
};

void profile(void){
	int op, arg;
	op = M[PC] & 255;
	arg = M[PC] >> 8;
	icnt[op]++;
	lcnt[PC]++;
//	if( op == icCall ) printf("%s ",findName(3,arg));
}
/*
void trace(void){
	while(runflag) {
//		printf("%d ",PC);
//		profile();
		run3();
		ninst++;
//		if(ninst > 5000000) break;
	}
}
*/
void outprof(void){
	int i, s;
	FILE *fo;
	s = 0;
	fo = fopen("prof.txt","w");
	for(i=1; i<=icSeqi; i++){
		fprintf(fo,"%s %d\n",opstr[i],icnt[i]);
		s += icnt[i];
	}
	fprintf(fo,"%d\n",s);
	for(i=1; i<CP; i++)
		fprintf(fo,"%d %d\n",i,lcnt[i]);
	fclose(fo);
}

/* alu operation input : a, b
 affect condition code Z,S
 carry and overflow are not implemented

 order of operand  ts = ff op ts
 ff is the first operand !
 */

int alu(int op, int a, int b){
  int t;
  switch(op) {
  case icAdd: t = b + a; break;
  case icSub: t = b - a; break;  // ordering!
  case icMul: t = b * a; break;
  case icDiv: t = b / a; break;
  case icBand: t = b & a; break;
  case icBor: t = b | a; break;
  case icBxor: t = b ^ a; break;
  case icNot: t = ! a; break;
  case icShl: t = b << a; break;
  case icShr: t = b >> a; break;
  case icEq: t = b == a; break;
  case icNe: t = b != a; break;
  case icLt: t = b < a; break;
  case icLe: t = b <= a; break;
  case icGt: t = b > a; break;
  case icGe: t = b >= a; break;
  case icInc: t = a + 1; break;
  case icDec: t = a - 1; break;
  case FSUB: t = a - b; break;
  case FP1: t = a; break;
  case FP2: t = b; break;
  case FZ: t = a == 0; break;
  }
  if( t == 0 ) Z = 1; else Z = 0;
  if( t < 0 )  S = 1; else S = 0;
  return t;
}

void initchip(void){
	clock = 0;
	ninst = 0;
	runflag = 1;
	TS = 0;
	NX = 0;
	FF = 0;
	FP = SSBASE;
	SP = SSBASE;
	PC = 1;
	Z = 0;
	S = 0;
	mpc = 0;
}

void yield(void){
	if(intflag && (clock-dt) > TIMEOUT)
		noss(time_out);
}

// processor execution
void eval(int ref){
	PC = ref;
	while(runflag ) {
		profile();
		run3();		// execute one instruction
		ninst++;
		yield();
	}
}

PRIVATE void popts(void){
	TS = M[SP];
	SP--;
}

void trap(int op, int arg){		// special functions
	switch(op) {
	case icEnd : noss(stopped);	break;
	case icArray: TS = xalloc(TS); break;
	case icSys:
		switch(arg){
		case 1: 				// print ts
			printf("%d",TS);
			popts();
			break;
		case 2 : 				// printc ts
			printf("%c",TS);
			popts();
			break;
		case 13: runflag = 0; break;
		case 20: intflag = 1; break;
		case 21: intflag = 0; break;
		case 22: noss(blocked); break;
		default:
			error("unknown syscall");
		}
		break;
	default:
		error("unknown trap");
	}
}

/*
int main(int argc, char *argv[]){
  if( argc < 2 ) {
    printf("usage : sx objfile\n");
    exit(0);
  }
  loadobj(argv[1]);
  readmspec();
  readmpgm();
//  printout();
  initrom();
//  showmxall();
  initchip();
//  interp();
  tok = xalloc(80);
  trace();
//  outprof();
  return 0;
}
*/
